/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.datamodel;

import static org.testng.AssertJUnit.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.testng.annotations.Test;

public class HiddenColumnsCursorTest
{
  /**
   * Test the method which finds the corresponding region given a column
   */
  @Test(groups = { "Functional" })
  public void testFindRegionForColumn()
  {
    HiddenColumnsCursor cursor = new HiddenColumnsCursor();
    
    int regionIndex = cursor.findRegionForColumn(20);
    assertEquals(-1, regionIndex);
    
    List<int[]> hidden = new ArrayList<>();
    hidden.add(new int[] { 53, 76 });
    hidden.add(new int[] { 104, 125 });
    cursor.resetCursor(hidden);

    regionIndex = cursor.findRegionForColumn(126);
    assertEquals(2, regionIndex);

    regionIndex = cursor.findRegionForColumn(125);
    assertEquals(1, regionIndex);

    regionIndex = cursor.findRegionForColumn(108);
    assertEquals(1, regionIndex);

    regionIndex = cursor.findRegionForColumn(104);
    assertEquals(1, regionIndex);

    regionIndex = cursor.findRegionForColumn(103);
    assertEquals(1, regionIndex);

    regionIndex = cursor.findRegionForColumn(77);
    assertEquals(1, regionIndex);

    regionIndex = cursor.findRegionForColumn(76);
    assertEquals(0, regionIndex);

    regionIndex = cursor.findRegionForColumn(53);
    assertEquals(0, regionIndex);

    regionIndex = cursor.findRegionForColumn(52);
    assertEquals(0, regionIndex);

    regionIndex = cursor.findRegionForColumn(0);
    assertEquals(0, regionIndex);
  }

  /**
   * Test the method which counts the number of hidden columns before a column
   */
  @Test(groups = { "Functional" })
  public void testGetHiddenOffset()
  {
    HiddenColumnsCursor cursor = new HiddenColumnsCursor();

    int offset = cursor.getHiddenOffset(20);
    assertEquals(-1, offset);

    List<int[]> hidden = new ArrayList<>();
    hidden.add(new int[] { 53, 76 });
    hidden.add(new int[] { 104, 125 });
    cursor.resetCursor(hidden);

    offset = cursor.getHiddenOffset(80);
    assertEquals(46, offset);

    offset = cursor.getHiddenOffset(79);
    assertEquals(24, offset);

    offset = cursor.getHiddenOffset(53);
    assertEquals(24, offset);

    offset = cursor.getHiddenOffset(52);
    assertEquals(0, offset);

    offset = cursor.getHiddenOffset(10);
    assertEquals(0, offset);

    offset = cursor.getHiddenOffset(0);
    assertEquals(0, offset);
  }

}
